/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup ble_sdk_uart_over_ble_main main.c
 * @{
 * @ingroup  ble_sdk_app_nus_eval
 * @brief    UART over BLE application main file.
 *
 * This file contains the source code for a sample application that uses the Nordic UART service.
 * This application uses the @ref srvlib_conn_params module.
 */

#include "main.h"

// @brief Function for initializing the timer module.
static void init_timers(void) {
  ret_code_t err_code = app_timer_init();
  APP_ERROR_CHECK(err_code);

  // Create timers.

  // TODO YOUR_JOB: Create any timers to be used by the application.
  //                Below is an example of how to create a timer.
  //                For every new timer needed, increase the value of the macro APP_TIMER_MAX_TIMERS by
  //                one.
  //      ret_code_t err_code;
  //      err_code = app_timer_create(&m_app_timer_id, APP_TIMER_MODE_REPEATED, timer_timeout_handler);
  //      APP_ERROR_CHECK(err_code);
}

// @brief Function for handling the YYY Service events.
// TODO YOUR_JOB implement a service handler function depending on the event the service you are using can generate
//
// @details This function will be called for all YY Service events which are passed to
//          the application.
//
// @param[in]   p_yy_service   YY Service structure.
// @param[in]   p_evt          Event received from the YY Service.
//
//
//static void on_yys_evt(ble_yy_service_t     * p_yy_service,
//                       ble_yy_service_evt_t * p_evt)
//{
//    switch (p_evt->evt_type)
//    {
//        case BLE_YY_NAME_EVT_WRITE:
//            APPL_LOG("[APPL]: charact written with value %s. ", p_evt->params.char_xx.value.p_str);
//            break;
//
//        default:
//            // No implementation needed.
//            break;
//    }
//}

// @brief Function for starting timers.
static void application_timers_start(void) {
  // TODO YOUR_JOB: Start your timers. below is an example of how to start a timer.
  //      ret_code_t err_code;
  //      err_code = app_timer_start(m_app_timer_id, TIMER_INTERVAL, NULL);
  //      APP_ERROR_CHECK(err_code);
}

// @brief Function for initializing buttons and leds.
//
// @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
static void init_buttons_leds(bool *p_erase_bonds) {
  bsp_event_t startup_event;

  uint32_t err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
  APP_ERROR_CHECK(err_code);

  err_code = bsp_btn_ble_init(NULL, &startup_event);
  APP_ERROR_CHECK(err_code);

  *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}

// @brief Function for initializing the nrf log module.
static void init_log(void) {
  ret_code_t err_code = NRF_LOG_INIT(NULL);
  APP_ERROR_CHECK(err_code);

  NRF_LOG_DEFAULT_BACKENDS_INIT();
}

// @brief Function for initializing power management.
static void init_power_management(void) {
  ret_code_t err_code;
  err_code = nrf_pwr_mgmt_init();
  APP_ERROR_CHECK(err_code);
}

// @brief Function for handling the idle state (main loop).
//
// @details If there is no pending log operation, then sleep until next the next event occurs.
static void idle_state_handle(void) {
  if (NRF_LOG_PROCESS() == false) {
    nrf_pwr_mgmt_run();
  }
  else {
    printf("NRF_LOG_PROCESS is TRUE");
  }
}

// @brief Application main function.
int main(void) {
  // Initialize.
  initialize();

  // Start execution.
  printf("\r\nUART started.\r\n");
  NRF_LOG_INFO("Debug logging for UART over RTT started.");
  advertising_start();

  // Enter main loop.
  for (;;) {
    idle_state_handle();
  }
}

// @brief Initialization
void initialize() {
  // Microcontroller initialization
  init_uCtrl();

  // Initialize the TWI/I2C Serial Communication
  init_twi();

  // BLE Stack initialization
  init_isp1807();
}

// @brief Sets the default value for the microcontroller
void init_uCtrl() {
  // TODO This is never set so not sure how or why it is used.
  bool erase_bonds;

  // Logger initialization
  init_log();
  
  // Timer initialization
  init_timers();

  // Button and LED initialization (not sure if does anything)
  init_buttons_leds(&erase_bonds);

  // Initialize the power management
  init_power_management();
}